package classicaltm.gui;

import java.awt.BorderLayout;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import classicaltm.TuringSimulator;
import classicaltm.model.TuringMachine;
import classicaltm.model.TuringProgram;

/**
 * The GUI for the Turing machine simulator.
 */
public class GUI extends JFrame {
	
	/**
	 * The editor tab of the GUI.
	 */
	private EditorTab editorTab;
	
	/**
	 * The simulator tab of the GUI.
	 */
	private SimulatorTab simulatorTab;
	
	/**
	 * Creates the GUI, and adds the sub-components to it.
	 */
	public GUI() {
		super("Classic Turing machine simulator (version " + TuringSimulator.VERSION + ")");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		setLayout(new BorderLayout());
		
		final JTabbedPane tabs = new JTabbedPane();
		editorTab = new EditorTab();
		tabs.addTab("Editor", editorTab);
		simulatorTab = new SimulatorTab();
		tabs.addTab("Simulator", simulatorTab);
		add(tabs, BorderLayout.CENTER);
		
		tabs.addChangeListener(new ChangeListener() {
			
			@Override
			public void stateChanged(ChangeEvent e) {
				if (tabs.getSelectedIndex() == 1) {
					String program = editorTab.getProgramText();
					TuringProgram tp;
					try {
						tp = Parser.parseProgram(program);
					} catch (ParseException ex) {
						// error while parsing
						tabs.setSelectedIndex(0);
						JOptionPane.showMessageDialog(GUI.this,
								"<html>Error while reading the Turing program:" +
								"<blockquote>Line <b>" + ex.getLineNumber() + "</b>: " +
								ex.getMessage() + "</blockquote>",
								"Parsing error",
								JOptionPane.ERROR_MESSAGE);
						return;
					}
					TuringMachine machine = new TuringMachine(tp);
					simulatorTab.setTuringMachine(machine);
				} else {
				    // ensure that the transition timer doesn't run anymore
				    simulatorTab.stopStepTimer();
				}
			}
		});
	}
	
	/**
	 * Returns the editor tab of the GUI.
	 * @return The editor tab.
	 */
	public EditorTab getEditorTab() {
		return editorTab;
	}
	
	/**
	 * Returns the simulator tab of the GUI.
	 * @return The simulator tab.
	 */
	public SimulatorTab getSimulatorTab() {
		return simulatorTab;
	}
}