package classicaltm.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;

/**
 * The toolbar that enables the user to specify the input to the simulation.
 */
public class OutputToolBar extends JToolBar {
	
	private SimulatorTab simulatorTab;
	private JTextField outputLabel;

    /**
     * Creates a new output toolbar.
     * @param s The simulator tab this toolbar is for.
     */
	public OutputToolBar(SimulatorTab s) {
		
		this.simulatorTab = s;
		
		setFloatable(false);
		
        add(new JLabel("Output: "));
        outputLabel = new JTextField(15);
        outputLabel.setEnabled(false);
        add(outputLabel);
        
        add(new JButton("Start again") {{
            addActionListener(new ActionListener() {
                
                @Override
                public void actionPerformed(ActionEvent e) {
                    
                    simulatorTab.getTuringMachine().reset();
                    simulatorTab.updateValues();
                    
                    // go to the input mode
                    simulatorTab.setMode(SimulatorTab.SimulatorMode.ASKING_INPUT);
                }
            });
        }});
	}
	
	/**
	 * Sets the output displayed in this tool bar.
	 * 
	 * @param text The output text.
	 */
	public void setOutputText(String text) {
        outputLabel.setText(text);
    }
}
