package classicaltm.gui;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.GridLayout;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import classicaltm.model.Transition;
import classicaltm.model.TuringMachine;

/**
 * A panel that shows the current state of the Turing machine, by showing
 * some textual values.
 */
public class TextProgramPanel extends JPanel {
	
    /**
     * The Turing machine to show the state of.
     */
	private TuringMachine machine;
	
	/**
	 * A label that shows the current state.
	 */
	private JLabel currentStateLabel;
	
    /**
     * A label that shows the next (active) transition.
     */
	private JLabel nextTransitionLabel;
	
    /**
     * A label that shows the transition count.
     */
	private JLabel stepCountLabel;
	
	/**
	 * Creates a new TextProgramPanel with no machine yet.
	 */
	public TextProgramPanel() {
		super(new BorderLayout(5, 5));
		
		this.machine = null;
		
		JPanel labelPanel = new JPanel(new GridLayout(1, 1, 5, 5));
		labelPanel.add(new JLabel("Current state:") {{ setHorizontalAlignment(CENTER); }});
		labelPanel.add(new JLabel("Next transition:") {{ setHorizontalAlignment(CENTER); }});
		labelPanel.add(new JLabel("Number of transitions taken:") {{ setHorizontalAlignment(CENTER); }});
		add(labelPanel, BorderLayout.PAGE_START);

		JPanel valuesPanel = new JPanel(new GridLayout(1, 1, 5, 5));
		currentStateLabel = new JLabel("-");
		currentStateLabel.setHorizontalAlignment(SwingConstants.CENTER);
		currentStateLabel.setFont(currentStateLabel.getFont().deriveFont(30f).deriveFont(Font.BOLD));
		valuesPanel.add(currentStateLabel);
		nextTransitionLabel = new JLabel("-");
		nextTransitionLabel.setHorizontalAlignment(SwingConstants.CENTER);
		nextTransitionLabel.setFont(nextTransitionLabel.getFont().deriveFont(30f).deriveFont(Font.BOLD));
		valuesPanel.add(nextTransitionLabel);
		stepCountLabel = new JLabel("-");
		stepCountLabel.setHorizontalAlignment(SwingConstants.CENTER);
		stepCountLabel.setFont(stepCountLabel.getFont().deriveFont(30f).deriveFont(Font.BOLD));
		valuesPanel.add(stepCountLabel);
		add(valuesPanel, BorderLayout.CENTER);
	}
	
	/**
	 * Sets the Turing machine to show in this panel.
	 * @param machine The Turing machine.
	 */
	public void setMachine(TuringMachine machine) {
		this.machine = machine;
	}
	
	/**
	 * Updates all of the values in this panel. This should be called when the
     * Turing machine changes its state (for example, when a step is executed).
	 */
	public void updateValues() {
		currentStateLabel.setText(machine == null ? "-" : machine.getCurrentState());
		
		if (machine != null) {
			Transition t = machine.activeTransition();
			
			nextTransitionLabel.setText(t == null ? "Done!" : t.toString());
		}
		
		stepCountLabel.setText(machine == null ? "-" : "" + machine.getStepCount());
	}
}
